// [Licence]
// The Ariba-Underlay Copyright
//
// Copyright (c) 2008-2009, Institute of Telematics, Universität Karlsruhe (TH)
//
// Institute of Telematics
// Universität Karlsruhe (TH)
// Zirkel 2, 76128 Karlsruhe
// Germany
//
// Redistribution and use in source and binary forms, with or without
// modification, are permitted provided that the following conditions are
// met:
//
// 1. Redistributions of source code must retain the above copyright
// notice, this list of conditions and the following disclaimer.
// 2. Redistributions in binary form must reproduce the above copyright
// notice, this list of conditions and the following disclaimer in the
// documentation and/or other materials provided with the distribution.
//
// THIS SOFTWARE IS PROVIDED BY THE INSTITUTE OF TELEMATICS ``AS IS'' AND
// ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
// IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
// PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE ARIBA PROJECT OR
// CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
// EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
// PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
// PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
// LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
// NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
// SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
//
// The views and conclusions contained in the software and documentation
// are those of the authors and should not be interpreted as representing
// official policies, either expressed or implied, of the Institute of
// Telematics.
// [Licence]

#ifndef ARIBA_BASE_MSG_H__
#define ARIBA_BASE_MSG_H__

#include <string>
#include <boost/cstdint.hpp>
#include "ariba/utility/messages.h"
#include "ariba/utility/serialization.h"
#include "ariba/utility/types/LinkID.h"
#include "ariba/utility/types/Address.h"
#include "ariba/utility/types/ServiceID.h"

using std::string;
using ariba::utility::Address;
using ariba::utility::LinkID;
using ariba::utility::Message;
using ariba::utility::ServiceID;

namespace ariba {
namespace communication {

using_serialization;

class AribaBaseMsg : public Message {
	VSERIALIZEABLE;
public:

	typedef enum _LINK_STATE {
		LINK_STATE_DATA          = 0,
		LINK_STATE_OPEN_REQUEST  = 1,
		LINK_STATE_OPEN_REPLY    = 2,
		LINK_STATE_CLOSE_REQUEST = 3, // there is no close reply. send request and local link is closed
		LINK_STATE_UPDATE        = 4,
	} LINK_STATE;

	AribaBaseMsg(	const Address* address    = NULL,
			LINK_STATE _state         = LINK_STATE_DATA,
			const LinkID& _localLink  = LinkID::UNSPECIFIED,
			const LinkID& _remoteLink = LinkID::UNSPECIFIED );

	virtual ~AribaBaseMsg();

	const LINK_STATE getType();
	const string getTypeString();
	const LinkID& getLocalLink();
	const LinkID& getRemoteLink();

private:
	uint8_t state;		// the link message type
	LinkID localLink;	// the local link id
	LinkID remoteLink;	// the remote link id

};

}} // ariba::communication

sznBeginDefault( ariba::communication::AribaBaseMsg, X ) {
	X && state && &localLink && &remoteLink && Payload();
} sznEnd();

#endif // ARIBA_BASE_MSG_H__
